const Lang = imports.lang;
const Applet = imports.ui.applet;
const GLib = imports.gi.GLib;


const Gettext = imports.gettext;
const _ = Gettext.gettext;

const Mainloop = imports.mainloop;
const Gtk = imports.gi.Gtk;

const Cairo = imports.cairo;
const St = imports.gi.St;
const Settings = imports.ui.settings;
const Config = imports.misc.config;
const Util = imports.misc.util;
const PopupMenu = imports.ui.popupMenu;

const Soup = imports.gi.Soup;
const Pango = imports.gi.Pango;

const _properies = {
	'title': 'No-IP Update Client',
	'cssname': 'applet-noip',
	'settings': 'cinnamon-settings applets %s',
	'useragent': 'noip@tuxnet24.de/1.0 info@tuxnet24.de',
	'config': [
		'apiurl',
		'username',
		'password',
		'ddnshost',
		'dateformat',
		'refreshInterval',
		'whitePanelIcons'
	],
	'debug': false
};


const _httpSession = new Soup.SessionAsync();
Soup.Session.prototype.add_feature.call(_httpSession, new Soup.ProxyResolverDefault())

function MyApplet(metadata, orientation, panel_height, instance_id) {

	this.uuid = metadata.uuid;
	this.path = metadata.path;
	this.version = metadata.version;
	this.maintainer = metadata.maintainer;

	this.settings = new Settings.AppletSettings(this, this.uuid, instance_id);

	this.iconpath = this.path + '/icons';
	Gtk.IconTheme.get_default().append_search_path(this.iconpath);

	// set locale path and gettext domain
	Gettext.bindtextdomain('messages', this.path + "/locale");
	Gettext.textdomain('messages');

	this._init(orientation, panel_height, instance_id);

}

MyApplet.prototype = {
	__proto__: Applet.TextIconApplet.prototype,

	_init: function(orientation, panel_height, instance_id) {
		Applet.TextIconApplet.prototype._init.call(this, orientation, panel_height, instance_id);

		// Status menu icon
		this.set_applet_icon_symbolic_name('applet-noip');
		this.set_applet_label("");
		this.set_applet_tooltip(_("Click to open"));

		// new PopupMenu object
		this.menuManager = new PopupMenu.PopupMenuManager(this);
		this.menu = new Applet.AppletPopupMenu(this, orientation);
		this.menu.actor.add_style_class_name(_properies.cssname + '-menu');
		this.menuManager.addMenu(this.menu);

		// bind properties
		for (let keys in _properies.config) {
			this.settings.bindProperty(
				Settings.BindingDirection.IN,
				_properies.config[keys],
				'_' + _properies.config[keys],
				this.refreshApplet,
				null);
		}

		this.settings.connect('changed::whitePanelIcons', Lang.bind(this, function() {
            this.set_applet_icon_symbolic_name('pannel-noip');
			this.refreshApplet(false);
		}));

		// configuration via context menu is automatically provided in Cinnamon 2.0+
		let cinnamonVersion = Config.PACKAGE_VERSION.split('.')
		let majorVersion = parseInt(cinnamonVersion[0])

		// for Cinnamon 1.x, build a menu item
		if (majorVersion < 2) {
			let itemLabel = _("Configure...");
			let settingsMenuItem = new Applet.MenuItem(itemLabel, Gtk.STOCK_EDIT, Lang.bind(this, function() {
				Util.spawnCommandLine(_properies.settings.replace(/%s/, this.uuid));
			}))
			this._applet_context_menu.addMenuItem(settingsMenuItem);
		}

		// main box
		let mainBox = new St.BoxLayout({ vertical: true });
		this.menu.addActor(mainBox);

		// top area
		this._topArea = new St.Bin({ style_class: _properies.cssname +'-toparea' });
		mainBox.add_actor(this._topArea);

		//  horizontal rule
		this._separatorArea = new St.DrawingArea({ style_class: 'popup-separator-menu-item' });
		this._separatorArea.width = 200;
		this._separatorArea.connect('repaint', Lang.bind(this, this._onSeparatorAreaRepaint));
		mainBox.add_actor(this._separatorArea);

		this._bottomArea = new St.Bin({ style_class: _properies.cssname + '-bottomarea' });
		mainBox.add_actor(this._bottomArea);

		this.rebuild();

		Mainloop.timeout_add_seconds(3, Lang.bind(this, function mainloopTimeout() {
			this.refreshApplet(true);
		}));

	},

	_log: function(message) {
		global.log(this.uuid + "#" + this._log.caller.name + ": " + message);
	},

	_onSeparatorAreaRepaint: function onSeparatorAreaRepaint(area) {
		let cr = area.get_context();
		let themeNode = area.get_theme_node();
		let [width, height] = area.get_surface_size();
		let margin = themeNode.get_length('-margin-horizontal');
		let gradientHeight = themeNode.get_length('-gradient-height');
		let startColor = themeNode.get_color('-gradient-start');
		let endColor = themeNode.get_color('-gradient-end');
		let gradientWidth = (width - margin * 2);
		let gradientOffset = (height - gradientHeight) / 2;
		let pattern = new Cairo.LinearGradient(margin, gradientOffset, width - margin, gradientOffset + gradientHeight);

		pattern.addColorStopRGBA(0, startColor.red / 255, startColor.green / 255, startColor.blue / 255, startColor.alpha / 255);
		pattern.addColorStopRGBA(0.5, endColor.red / 255, endColor.green / 255, endColor.blue / 255, endColor.alpha / 255);
		pattern.addColorStopRGBA(1, startColor.red / 255, startColor.green / 255, startColor.blue / 255, startColor.alpha / 255);
		cr.setSource(pattern);
		cr.rectangle(margin, gradientOffset, gradientWidth, gradientHeight);
		cr.fill();
	},

	updateIconType: function() {

		this._icon_type = this.settings.getValue('useSymbolicIcons') ?
			St.IconType.SYMBOLIC :
			St.IconType.FULLCOLOR;

	},

	rebuild: function() {
		this.loadingUi();
		this.rebuildTopUi();
		this.rebuildBottomUi();
	},

	loadingUi: function() {
		this.destroyTopArea();
		this.destroyBottomArea();
		this._topArea.set_child(new St.Label({    text: _('Loading, prease wait ...') }));
		this._bottomArea.set_child(new St.Label({ text: _('Applet loading, prease wait ...') }));
	},

	rebuildTopUi: function() {

		this.destroyTopArea();

		// Box Icon
		this._appletIcon = new St.Icon({
			icon_type: St.IconType.SYMBOLIC,
			icon_size: 64,
			icon_name: 'applet-noip',
			style_class: _properies.cssname + '-icon'
		});

		// Box Key
		let boxkey = new St.BoxLayout({
			vertical: true,
			style_class: _properies.cssname + '-box-key'
		});

		this._keyLabel1 = new St.Label({
			text: _('Domain'),
			style_class: _properies.cssname + '-key-label'
		});
		this._keyLabel2 = new St.Label({
			text: _('IP-Address'),
			style_class: _properies.cssname + '-key-label'
		});
		this._keyLabel3 = new St.Label({
			text: _('Timestamp'),
			style_class: _properies.cssname + '-key-label'
		});
		boxkey.add_actor(this._keyLabel1);
		boxkey.add_actor(this._keyLabel2);
		boxkey.add_actor(this._keyLabel3);

		// Box Value
		let boxvalue = new St.BoxLayout({
			vertical: true,
			style_class: _properies.cssname + '-box-value'
		});

		this._valLabelDomain = new St.Label({
			text: _('..'),
			style_class: _properies.cssname + '-val-label'
		});
		this._valLabelIpaddr = new St.Label({
			text: _('..'),
			style_class: _properies.cssname + '-val-label'
		});
		this._valLabelTimestamp = new St.Label({
			text: _('..'),
			style_class: _properies.cssname + '-val-label'
		});
		boxvalue.add_actor(this._valLabelDomain);
		boxvalue.add_actor(this._valLabelIpaddr);
		boxvalue.add_actor(this._valLabelTimestamp);

		// Box Keys + Values
		let boxcontainer = new St.BoxLayout();
		boxcontainer.add_actor(boxkey);
		boxcontainer.add_actor(boxvalue);

		// Box button
		let boxbutton = new St.BoxLayout({
			vertical: true,
			style_class: _properies.cssname + '-box-button'
		});

		this._buttonIcon = new St.Button({
			reactive: true,
			style_class: _properies.cssname + '-icon-button'
		});
		boxbutton.add_actor(this._buttonIcon);

		// link to reload this applet
		this._buttonIcon.connect('clicked', Lang.bind(this, function() {
			this.refreshApplet(false);
            this.menu.toggle();
		}));

		// Box title
		let boxtitle = new St.BoxLayout({
			vertical: true,
			style_class: _properies.cssname + '-box-title'
		});

		this._appletTitle = new St.Label({
			text: _('No title'),
			style_class: _properies.cssname + '-title-label'
		});
		boxtitle.add_actor(this._appletTitle);
		boxtitle.add_actor(boxcontainer);

		// Box icon
		let boxicon = new St.BoxLayout({
			style_class: _properies.cssname + '-box-icon'
		});
		boxicon.add_actor(this._appletIcon);
		boxicon.add_actor(boxtitle);
		boxicon.add_actor(boxbutton);

		this._topArea.set_child(boxicon);

	},

	rebuildBottomUi: function() {

		this.destroyBottomArea();

		let box = new St.BoxLayout({
			vertical: true,
			style_class: _properies.cssname + '-box-status'
		});

		this._appletMessage = new St.Label({
			text: _('Applet loading, prease wait ...'),
			style_class: _properies.cssname + '-label-message'
		});
		this._appletMessage.clutter_text.line_wrap = true;
		this._appletMessage.clutter_text.line_wrap_mode = Pango.WrapMode.WORD;
		this._appletMessage.clutter_text.ellipsize = Pango.EllipsizeMode.NONE;
		box.add_actor(this._appletMessage);

		let scrollbox = new St.ScrollView({ x_fill: true, y_fill: false, y_align: St.Align.START, style_class: 'vfade menu-applications-scrollbox' });
		scrollbox.add_actor(box);

		this._bottomArea.set_child(scrollbox);

	},

	destroyTopArea: function() {
		if (this._topArea.get_child() != null)
			this._topArea.get_child().destroy();
	},

	destroyBottomArea: function() {
		if (this._bottomArea.get_child() != null)
			this._bottomArea.get_child().destroy();
	},

	refreshApplet: function(recurse) {

		// Get the NAT ipaddress
		let ipaddr = this.getIPaddr();

		try {

			let appleticon = 'applet-noip';
			let panelicon = (this._whitePanelIcons ? 'pannel-noip' : 'applet-noip');
			let applettooltip = _('Click to open');
			let appletmessage = _('Applet loading, prease wait ...');

			// Update no-ip host and get the responce
			let content = this.updateNoipHost(this.apiUrl(ipaddr));

			if (content['status']) {
				if (content['status'] === 'good' || content['status'] === 'nochg') {
					appleticon = 'applet-noip';
					panelicon = (this._whitePanelIcons ? 'pannel-noip' : 'applet-noip');
					applettooltip = _('IP-Address') + ': ' + ipaddr;
				} else {
					appleticon = 'applet-noip-error';
					panelicon = (this._whitePanelIcons ? 'pannel-noip-error' : 'applet-noip-error');
					applettooltip = _('ERROR: Click to open for details');
				}

				appletmessage = content['msg'];
			}

			// Applet Icon
			this._appletIcon.icon_name = appleticon;

			// Applet Title
			this._appletTitle.text = _(_properies.title);

			// Applet Values
			this._valLabelDomain.text = this._ddnshost;
			this._valLabelIpaddr.text = ipaddr;
			this._valLabelTimestamp.text = this.showDate();

			// Applet Status Message
			this._appletMessage.text = appletmessage;

			// Panel Icon
			this._icon_type = this.set_applet_icon_symbolic_name(panelicon);
			this.set_applet_label("");
			this.set_applet_tooltip(applettooltip);

		}
		catch (e) {
			global.logError(e);
		}

		if (recurse) {
			Mainloop.timeout_add_seconds(this._refreshInterval * 60, Lang.bind(this, function() {
				this.refreshApplet(true);
			}));
		}

	},

	isipaddr: function (ipaddress) {

		return (/^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/.test(ipaddress)); 

	},

	showDate: function () {

		let date = new Date();
		return date.toLocaleFormat(this._dateformat);

	},

	apiUrl: function (ip) {
		let apiurl = this._apiurl.replace(/^(https?:\/\/)(\S+)/g, '$1'+this._username+':'+this._password+'@$2');
		let output = apiurl+'?hostname=' + this._ddnshost + '&myip='+ip;
		if (_properies.debug) this._log('request_uri=' + output);
		return output
	},

	getNatIp: function (url) {

		let ip = null;
		let message = Soup.Message.new('GET', url);

		// Set user-agent
		_httpSession['user-agent'] = _properies.useragent;

		_httpSession.send_message(message);
		if (message.status_code === 200) {
			let body = message.response_body.data.toString();
			let regex = /([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3})/g;
			let match = regex.exec(body);
			if (match[1]) ip = match[1];
		}

		return ip;

	},

	getIPaddr: function () {

		var ip=null;
		var counter=0;
		while (!this.isipaddr(ip)) {
			switch (counter) {
				case 0:
					ip = this.getNatIp('http://natip.tuxnet24.de/index.php?mode=text');
					if (_properies.debug) this._log('page=http://natip.tuxnet24.de/index.php?mode=text; ip=' + ip);
					counter++;
				break;
				case 1:
					ip = this.getNatIp('http://checkip.dyndns.org');
					if (_properies.debug) this._log('page=http://checkip.dyndns.org; ip=' + ip);
					counter++;
				break;
				case 2:
					ip = this.getNatIp('http://wtfismyip.com/text');
					if (_properies.debug) this._log('page=http://wtfismyip.com/text; ip=' + ip);
					counter++;
				break;
				case 3:
					ip = this.getNatIp('http://icanhazip.com');
					if (_properies.debug) this._log('page=http://icanhazip.com; ip=' + ip);
					counter++;
				break;
				case 4:
					ip = this.getNatIp('http://www.networksecuritytoolkit.org/nst/tools/ip.php');
					if (_properies.debug) this._log('page=http://www.networksecuritytoolkit.org/nst/tools/ip.php; ip=' + ip);
					counter++;
				break;
				default:
					void(0); // exit...?
				break;
			}
		}
		return ip;

	},

	updateNoipHost: function(url) {

		let data = new Array();
		let message = Soup.Message.new('GET', url);

		// Set user-agent
		_httpSession['user-agent'] = _properies.useragent;

		_httpSession.send_message(message);
		if (message.status_code === 200) {
			let content = message.response_body.data.toString().split(" ");
			if (content[0]) {
				data['status'] = content[0];
				data['msg'] = this.getStatusMessage(content[0]);
			}
		}

		return data;

	},

	getStatusMessage: function(state) {

		let msg=null;
		switch (state) {
			case 'good':
				msg = _("DNS hostname %s update successful.").replace(/%s/, this._ddnshost);
			break;
			case 'nochg':
				msg = _("IP address is current, no update performed.");
			break;
			case 'nohost':
				msg = _("Hostname supplied does not exist under specified account.");
			break;
			case 'badauth':
				msg = _("Invalid username password combination.");
			break;
			case 'badagent':
				msg = _("Client disabled. Client should exit and not perform any more updates without user intervention.");
			break;
			case '!donator':
				msg = _("An update request was sent including a feature that is not available to that particular user such as offline options.");
			break;
			case 'abuse':
				msg = _("Username is blocked due to abuse.");
			break;
			case '911':
				msg = _("A fatal error on our side such as a database outage. Retry the update no sooner than 30 minutes.");
			break;
			default:
				msg = _("The return value from no-ip.com can not be evaluated.");
			break;
		}

		if (_properies.debug) this._log('state=' + state + '; msg=' + msg);

		return msg;

	},

	on_applet_clicked: function(event) {
		this.menu.toggle();
	}

};

function main(metadata, orientation, panel_height, instance_id) {
	return new MyApplet(metadata, orientation, panel_height, instance_id);
}

